package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.projectlib.model.entity.MhProject;
import com.hz.pm.api.projectlib.model.entity.MhProjectSchemaTargetData;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.req.DeclaredProjectListReq;
import com.hz.pm.api.projectlib.model.vo.DeclaredProjectListVO;
import com.hz.pm.api.projectlib.model.vo.MhProjectBaseInfoVO;
import com.hz.pm.api.projectlib.service.IMhProjectSchemaTargetDataService;
import com.hz.pm.api.projectlib.service.IMhProjectService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.dto.UnitDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * MhProjectManage
 * </p>
 *
 * @author WendyYang
 * @since 19:06 2024/1/30
 */
@Component
@RequiredArgsConstructor
public class MhProjectManage {

    private final IMhProjectService mhProjectService;
    private final IMhProjectSchemaTargetDataService mhProjectSchemaTargetDataService;
    private final IProjectService projectService;
    private final MhUnitCache mhUnitCache;

    private LambdaQueryWrapper<MhProject> buildQuery(DeclaredProjectListReq req) {
        LambdaQueryWrapper<MhProject> query = Wrappers.lambdaQuery(MhProject.class)
                .like(StrUtil.isNotBlank(req.getProjectName()), MhProject::getProjectName, req.getProjectName())
                .like(StrUtil.isNotBlank(req.getDeclaredUnit()), MhProject::getUnitName, req.getDeclaredUnit());
        if (req.getBudgetYear() != null) {
            LocalDateTime startTime = LocalDate.of(req.getBudgetYear(), 1, 1).atStartOfDay();
            LocalDateTime endTime = startTime.plusYears(1);
            query.ge(MhProject::getCreateTime, startTime);
            query.lt(MhProject::getCreateTime, endTime);
        }
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        if (!user.getSuperAdmin() && !user.getRegionAdmin()) {
            query.eq(MhProject::getUnitId, user.getMhUnitId());
        }
        return query;
    }

    public PageVo<DeclaredProjectListVO> pageDeclaredProject(DeclaredProjectListReq req) {
        LambdaQueryWrapper<MhProject> query = buildQuery(req);
        Page<MhProject> page = mhProjectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<MhProject> records = page.getRecords();
        List<String> projectCodes = CollUtils.fieldList(records, MhProject::getProjectCode);
        Map<String, Project> projectMap;
        if (!projectCodes.isEmpty()) {
            List<Project> projects = projectService.listNewestByProjectCodes(projectCodes);
            projectMap = CollUtils.listToMap(projects, Project::getProjectCode);
        } else {
            projectMap = Collections.emptyMap();
        }
        List<DeclaredProjectListVO> data = records.stream().map(w -> {
            DeclaredProjectListVO item = new DeclaredProjectListVO();
            item.setMhProjectId(w.getId());
            Project project = projectMap.get(w.getProjectCode());
            if (project == null) {
                item.setStatus(null);
                item.setStage(null);
            } else {
                item.setStatus(project.getStatus());
                item.setStage(project.getStage());
                item.setBizDomain(project.getBizDomain());
            }
            item.setProjectCode(w.getProjectCode());
            item.setProjectName(w.getProjectName());
            item.setDeclaredAmount(w.getTotalMoney());
            item.setCreateTime(w.getCreateTime());
            return item;
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

    public MhProjectBaseInfoVO getProjectBaseInfo(Long mhProjectId) {
        MhProject mhProject = mhProjectService.getById(mhProjectId);
        if (mhProject == null) {
            throw BizException.wrap("项目不存在");
        }
        MhProjectBaseInfoVO detail = new MhProjectBaseInfoVO();
        detail.setMhProjectId(mhProject.getId());
        detail.setProjectName(mhProject.getProjectName());
        detail.setDeclaredUnit(mhProject.getUnitName());
        detail.setDeclaredUnitId(mhProject.getUnitId());
        detail.setTotalInvestAmount(mhProject.getTotalMoney());
        detail.setReviewInvestAmount(mhProject.getMoney());
        MhProjectSchemaTargetData schemaTargetData = mhProjectSchemaTargetDataService.getByMhProjectId(mhProjectId);
        if (schemaTargetData != null) {
            detail.setTotalInvestBudgetAmount(schemaTargetData.getMoney());
        }
        UnitDTO unit = mhUnitCache.getById(mhProject.getId());
        if (unit != null) {
            detail.setDeclaredUnitUnifiedSocialCreditCode(unit.getUnifiedSocialCreditCode());
        }
        return detail;
    }


}
